// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz
// or its licensors, as applicable.
//
// You may not use this file except under the terms of the accompanying license.
//
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Project: tesseract-interface
// File: tesseract.h
// Purpose: interfaces to Tesseract
// Responsible: mezhirov
// Reviewer:
// Primary Repository:
// Web Sites: www.iupr.org, www.dfki.de


#ifndef h_tesseract_
#define h_tesseract_

#include "colib.h"
#include "regionextractor.h"

namespace ocropus {

    // A direct interface to Tesseract to bypass oversegmentation.
    // (no adaptation performed here; it's all through ITrainChars.)
    struct ISimpleLineOCR : colib::IComponent {
        // recognize the text line given by the binary image bimage
        virtual void recognize_binary(colib::nustring &result,
                                      colib::floatarray &costs,
                                      colib::narray<colib::rectangle> &bboxes,
                                      colib::bytearray &bimage) = 0;
        // recognize the text line given by the gray scale image image
        virtual void recognize_gray(colib::nustring &result,
                                    colib::floatarray &costs,
                                    colib::narray<colib::rectangle> &bboxes,
                                    colib::bytearray &image) = 0;
        // FIXME maybe add a "get character training images" method here
    };
    
    
    /// \brief Make a wrapper for Tesseract.
    /// \warning There's really only one Tesseract,
    /// so instances would train each other!
    ///     (but this is not a singleton so that autodel<>
    ///      can be used normally).
    ///
    /// The returned bounding boxes may be 1-2 pixel inaccurate
    /// due to Tesseract's polygonal approximaion.
    ///
    /// The same costs are returned for every character in a word,
    /// because the Tesseract returns a struct
    /// that contains only the whole word's rating.
    ///
    /// @param path_to_our_executable Path to the executable (just argv[0]).
    ///                               Used to find tessdata/ directory.
    ISimpleLineOCR *make_tesseract(const char *path_to_our_executable = "");
    colib::IRecognizeLine *make_TesseractRecognizeLine();
    colib::ITrainChars *make_TesseractTrainChars();


    void tesseract_recognize_blockwise_and_dump(colib::bytearray &gray,
                                                colib::intarray &pageseg);

    void tesseract_recognize_blockwise_and_split_to_lines(
            colib::narray<colib::nustring> &text,
            colib::narray<colib::narray<colib::rectangle> > &bboxes,
            colib::narray<colib::floatarray> &costs,
            colib::bytearray &gray,
            colib::intarray &pseg);
};

#endif
