// Copyright 2006-2007 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: ocr-utils
// File: report.cc
// Purpose: HTML report back-end (output mixing text and images)
// Responsible: mezhirov
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "colib.h"
#include "imgio.h"
#include "report.h"

using namespace colib;

using namespace imgio;


namespace ocropus {

struct HTMLReport : IReport {
    stdio file;
    strbuf img_prefix;
    int images_written;

    HTMLReport(const char *path) : images_written(0) {
        file = fopen(path, "w");
        img_prefix.ensure(strlen(path) + 20);
        strcpy(img_prefix, path);
        strcat(img_prefix, "_image");
    }

    virtual void add(const char *format, ...) {
        va_list va;
        va_start(va, format);
        vfprintf(file, format, va);
        va_end(va);
    }

    virtual void add(bytearray &image) {
        narray<char> path(strlen(img_prefix) + 20);
        sprintf(&path[0], "%s%d.png", (char *) img_prefix, ++images_written);
        write_png(stdio(&path[0], "wb"), image);
        fprintf(file, "<IMG SRC=\"%s\">\n", &path[0]);
    }

    virtual void add(intarray &image) {
        narray<char> path(strlen(img_prefix) + 20);
        sprintf(&path[0], "%s%d.png", (char *) img_prefix, ++images_written);
        write_png_rgb(stdio(&path[0], "wb"), image);
        fprintf(file, "<IMG SRC=\"%s\">\n", &path[0]);
    }

    virtual void p() {
        fprintf(file, "<P>\n");
    }
};
IReport *make_HTML_report(const char *path) {
    return new HTMLReport(path);
}

};
